<?php

/*
 *  This file is part of SplashSync Project.
 *
 *  Copyright (C) 2015-2020 Splash Sync  <www.splashsync.com>
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *  For the full copyright and license information, please view the LICENSE
 *  file that was distributed with this source code.
 */

namespace Splash\Tests\Tools\Traits;

use ArrayObject;
use Splash\Client\Splash;

/**
 * Splash Test Tools - Objects Faker trait
 *
 * @author SplashSync <contact@splashsync.com>
 */
trait ObjectsFakerTrait
{
    /**
     * Object original Data
     * Used to Protect Required & NotTested fields from Update
     *
     * @var null|array
     */
    protected $originData;

    //==============================================================================
    //      FAKE DATA GENERATORS
    //==============================================================================

    /**
     * Generate Fake Object Fields List
     *
     * @param string      $objectType Object Type Name
     * @param array|false $fieldsList Object Field Ids List
     * @param bool        $associate  Include Associated Fields
     * @param bool        $nonTested  Include Non Tested Fields
     *
     * @return array $Out            Array of Fields
     */
    public function fakeFieldsList($objectType, $fieldsList = false, $associate = false, $nonTested = true)
    {
        //====================================================================//
        // Safety Check => $ObjectType is a valid
        $this->assertTrue(
            in_array($objectType, Splash::objects(), true),
            "Invalid Object Type Name. (".$objectType.")"
        );

        //====================================================================//
        // Create Empty Object Data Array
        $outputs = array();

        //====================================================================//
        // Load Object Fields Definition
        $fields = Splash::object($objectType)->fields();
        if (empty($fields)) {
            return $outputs;
        }

        //====================================================================//
        // Generate Fields Data
        foreach ($fields as $field) {
            //====================================================================//
            // Check if Fields is Needed
            if (!$this->isFieldNeeded($field, $fieldsList, $nonTested)) {
                continue;
            }
            //====================================================================//
            // Add Fields to List
            $outputs[$field->id] = $field;
        }

        //====================================================================//
        // No Associated Fields
        if (!$associate) {
            return $outputs;
        }

        //====================================================================//
        // Add Associated Fields to List
        foreach ($outputs as $outField) {
            //====================================================================//
            // No Associated Field
            if (empty($outField->asso)) {
                continue;
            }
            //====================================================================//
            // For Associated Fields
            foreach ($fields as $field) {
                if (in_array($field->id, self::toArray($outField->asso), true)) {
                    $outputs[$field->id] = $field;
                }
            }
        }

        return $outputs;
    }

    /**
     * Create Fake/Dummy Object Data
     *
     * @param array $fieldsList Object Field List
     *
     * @return array
     */
    public function fakeObjectData($fieldsList)
    {
        //====================================================================//
        // Create Dummy Data Array
        $outputs = array();
        if (empty($fieldsList)) {
            return $outputs;
        }

        //====================================================================//
        // Create Dummy Fields Data
        foreach ($fieldsList as $field) {
            //====================================================================//
            // Generate Single Fields Dummy Data (is Not a List Field)
            if (!self::isListField($field->id)) {
                $choices = self::toArray($field->choices);
                $options = self::toArray($field->options);
                $outputs[$field->id] = $this->isFieldToPreserve($field)
                    ? (isset($this->originData) ? $this->originData[$field->id] : null)
                    : self::fakeFieldData($field->type, $choices, $options);

                continue;
            }

            //====================================================================//
            // Generate Dummy List  Data
            $list = self::isListField($field->id);
            $this->assertIsArray($list);
            $listName = $list["listname"];
            $fieldName = $list["fieldname"];
            $listData = self::fakeListData($field);
            //====================================================================//
            // Create List
            if (!array_key_exists($listName, $outputs)) {
                $outputs[$listName] = array();
            }
            //====================================================================//
            // Parse Data in List
            foreach ($listData as $key => $data) {
                if (!array_key_exists($key, $outputs[$listName])) {
                    $outputs[$listName][$key] = array();
                }
                $outputs[$listName][$key][$fieldName] = $data[$fieldName];
            }
        }

        return $outputs;
    }

    /**
     * Create Fake/Dummy Object List Data
     *
     * @param ArrayObject $field Object Field Definition
     *
     * @return array
     */
    public function fakeListData($field)
    {
        //====================================================================//
        // Read Number of Items to Put in Lists
        $nbItems = $this->settings["ListItems"]?$this->settings["ListItems"]:2;
        //====================================================================//
        // Parse List Identifiers
        $list = self::isListField($field->id);
        $type = self::isListField($field->type);
        $this->assertIsArray($list);
        $this->assertIsArray($type);

        //====================================================================//
        // Generate Unik Dummy Fields Data
        $listData = array();
        $nbTry = 0;
        while (count($listData) < $nbItems) {
            //====================================================================//
            // Generate Dummy Fields Data
            $data = self::fakeFieldData(
                $type["fieldname"],
                self::toArray($field->choices),
                self::toArray($field->options)
            );
            //====================================================================//
            // Compute Md5
            // or Use Try Count In Case we were not able to generate Unik Data
            $md5 = ($nbTry < 10) ? md5(serialize($data)) : $nbTry;
            $listData[$md5] = $data;
            $nbTry++;
        }
        //====================================================================//
        // Data Set is Not Unik => Add A Warning
        if ($nbTry >= 10) {
            print_r("Unable to Generate Unik List Dataset from Field: ".$field->id.PHP_EOL);
            print_r("Generated List may Contain Duplicated Values".PHP_EOL);
            print_r("Possible Fix: Ensure Pointed Object List is NOT Empty".PHP_EOL);
        }

        //====================================================================//
        // Create Dummy List Data
        $outputs = array();

        //====================================================================//
        // Create Dummy Fields Data
        for ($i = 0; $i < $nbItems; $i++) {
            $outputs[][$list["fieldname"]] = array_shift($listData);
        }

        return $outputs;
    }

    /**
     * Create Fake Field data
     *
     * @param string $type    Object Field Type
     * @param array  $choices Object Field Possible Values
     * @param array  $options Object Field Values Options
     *
     * @return array|bool|string
     */
    public function fakeFieldData($type, $choices = null, $options = array())
    {
        //====================================================================//
        // Safety Check
        if (empty($type)) {
            return false;
        }
        //====================================================================//
        // Verify Field Type is Valid
        $className = self::isValidType($type);
        if (false == $className) {
            return false;
        }
        //====================================================================//
        // Detects Id Fields    => Cannot Generate Fake for Id Fields Here...
        if (($id = self::isIdField($type))) {
            return $className::fake(array_replace_recursive($this->settings, $options), $id["ObjectType"]);
        }

        //====================================================================//
        // Take Values From Given Choices
        if (!empty($choices)) {
            // Ensure Choices have numeric Index
            $choices = array_values($choices);
            // Select a Random Index
            $index = mt_rand(0, count($choices) - 1);
            if (isset($choices[$index]["key"]) && (SPL_T_VARCHAR == $type)) {
                return (string) $choices[$index]["key"];
            }
            if (isset($choices[$index]["key"])) {
                return $choices[$index]["key"];
            }
        }

        //====================================================================//
        // Generate Single Field Data Type is Valid
        return $className::fake(array_replace_recursive($this->settings, $options));
    }

    /**
     * Check if Field Need to be in List
     *
     * @param ArrayObject $field      Field Definition
     * @param array|false $fieldsList Object Field Ids List
     * @param bool        $nonTested  Include Non Tested Fields
     *
     * @return bool
     */
    private function isFieldNeeded($field, $fieldsList = false, $nonTested = true)
    {
        //====================================================================//
        // Check if Fields is Writable
        if (!$field->write) {
            return false;
        }
        //====================================================================//
        // Check if Fields is Needed
        //====================================================================//

        //====================================================================//
        // Required Field
        if ($field->required) {
            return true;
        }
        //====================================================================//
        // Non Tested Field
        if ((true == $field->notest) && (false == $nonTested)) {
            return false;
        }
        //====================================================================//
        // If NO Fields List is Given => Select All Write Fields
        if ((false == $fieldsList) || !is_array($fieldsList)) {
            return true;
        }
        //====================================================================//
        // Field is in Requested List
        if (!in_array($field->id, $fieldsList, true)) {
            return false;
        }

        return true;
    }

    /**
     * Check if Field Need to e in List
     *
     * @param ArrayObject $field Field Definition
     *
     * @return bool
     */
    private function isFieldToPreserve($field)
    {
        //====================================================================//
        // Check if Origin Data Exists
        if (!is_array($this->originData)) {
            return false;
        }
        //====================================================================//
        // Check if Origin Data Exists
        if (!isset($this->originData[$field->id]) || empty($this->originData[$field->id])) {
            return false;
        }
        //====================================================================//
        // Check if Fields Should be Tested or Not
        if (!$field->notest) {
            return false;
        }

        return true;
    }
}
